var container, camera, scene, renderer, geometry, material, animate, controls, helper, sphereLightMesh, spotLight, rotateAngle = 0; //常用变量
var clock = new THREE.Clock();

Ammo().then(function () {
  init();
  animate();
});

function init() {
  container = document.getElementById('demo');
  scene = new THREE.Scene();
  scene.background = new THREE.Color('#ffffff');
  camera = new THREE.PerspectiveCamera(50, window.innerWidth / window.innerHeight, 0.1, 1000);
  camera.position.set(0, 50, 90);

  lights(); //灯光：聚光灯
  plane(); //地面
  loadModel(); //添加人物模型
  rendererScene(); //场景渲染
  OrbitControls(camera, renderer); // 控制模块
  window.addEventListener('resize', onWindowResize, false); // 屏幕变化
}

function plane() {
  // BoxGeometry( width, height, depth, widthSegments, heightSegments, depthSegments )
  geometry = new THREE.BoxGeometry(80, 3, 80, 1, 1, 1);
  let matStdObjects = new THREE.MeshStandardMaterial({
    color: '#ffffff',
    roughness: 0,
    metalness: 0.6
  });
  mshStdBox = new THREE.Mesh(geometry, matStdObjects);
  mshStdBox.position.set(0, -3, 0);
  mshStdBox.rotation.set(0, Math.PI / 2, 0);
  mshStdBox.castShadow = true;
  mshStdBox.receiveShadow = true;
  scene.add(mshStdBox);
}

function lights() {

  let ambient = new THREE.AmbientLight('#ffffff', 0.2);
  scene.add(ambient);

  //  SpotLight( color：颜色, intensity：强度, distance：发光距离, angle：角度, penumbra：边缘范围, decay：衰减 )
  spotLight = new THREE.SpotLight('#ffffff', 1, 100);
  spotLight.position.set(-10, 40, 10);
  spotLight.angle = Math.PI / 4;

  spotLight.penumbra = 0.05; //边缘范围，反比
  spotLight.decay = 0.1; //衰减系数，反比
  spotLight.distance = 80; //发光距离
  spotLight.castShadow = true; //阴影
  spotLight.shadow.mapSize.width = 300;
  spotLight.shadow.mapSize.height = 300;
  spotLight.shadow.camera.near = 10; //近截面
  spotLight.shadow.camera.far = 120;
  scene.add(spotLight);

}

function loadModel() {

  helper = new THREE.MMDAnimationHelper({
    afterglow: 2.0
  });
  var loader = new THREE.MMDLoader();

  var modelFile = './model/miku_v2.pmd';
  var vmdFiles = ['./model/wavefile_v2.vmd'];

  loader.loadWithAnimation(modelFile, vmdFiles, function (mmd) {

    mesh = mmd.mesh;
    helper.add(mesh, {
      animation: mmd.animation,
      physics: true
    });
    mmd.mesh.traverse(function (child) {
      if (child.isMesh) {
        child.castShadow = true;
        child.receiveShadow = true;
      }
    });

    mesh.position.y = -1.5;
    scene.add(mesh);

  }, onProgress, null);

  function onProgress(xhr) {
    if (xhr.lengthComputable) {
      let percentComplete = xhr.loaded / xhr.total * 100;
      console.log(Math.round(percentComplete, 2) + '% downloaded');
    }
  }
}

function onWindowResize() {
  camera.aspect = window.innerWidth / window.innerHeight;
  camera.updateProjectionMatrix();
  renderer.setSize(window.innerWidth, window.innerHeight);
}

function rendererScene() {
  renderer = new THREE.WebGLRenderer({
    antialias: true
  });
  renderer.setPixelRatio(window.devicePixelRatio);
  renderer.setSize(window.innerWidth, window.innerHeight);
  renderer.shadowMap.enabled = true;
  renderer.shadowMap.type = THREE.PCFSoftShadowMap;
  renderer.gammaInput = true;
  renderer.gammaOutput = true;
  container.appendChild(renderer.domElement);
}

function OrbitControls(camera, renderer) {
  //OrbitControls控件操作模块
  controls = new THREE.OrbitControls(camera, renderer.domElement);
  controls.autoRotate = false; //是否自动旋转
  controls.autoRotateSpeed = 1; //自动旋转速度，正比
}

function animate() {
  requestAnimationFrame(animate);
  if (controls) {
    controls.update();
  }

  pointLightAnimation();

  helper.update(clock.getDelta());
  renderer.render(scene, camera);
};

function pointLightAnimation() {
  if (rotateAngle > 2 * Math.PI) {
    rotateAngle = 0 
  } else {
    rotateAngle += 0.02 
  }

  // 光源 y 轴旋转
  spotLight.position.x = 10 * Math.cos(rotateAngle);
  spotLight.position.z = 10 * Math.sin(rotateAngle);
}